@@ -23,14 +23,15 @@
 #include "FeaturedModel.h"
 #include "CachedNetworkAccessManager.h"
 #include "DiscoverDeclarativePlugin.h"
+#include "DiscoverBackendsFactory.h"
 
 // Qt includes
 #include <QAction>
 #include "discover_debug.h"
 #include <QDesktopServices>
-#include <QtQml/QQmlEngine>
-#include <QtQml/QQmlContext>
-#include <QtQml/QQmlApplicationEngine>
+#include <QQmlEngine>
+#include <QQmlContext>
+#include <QQmlApplicationEngine>
 #include <QtQuick/QQuickItem>
 #include <qqml.h>
 #include <QPointer>
@@ -51,7 +52,7 @@
 #include <KSharedConfig>
 #include <KConfigGroup>
 #include <KStandardAction>
-#include <KIO/AccessManager>
+#include <kcoreaddons_version.h>
 // #include <KSwitchLanguageDialog>
 
 // DiscoverCommon includes
@@ -68,6 +69,8 @@
 #include <utils.h>
 #include <QMimeDatabase>
 
+#include "discoversettings.h"
+
 class OurSortFilterProxyModel : public QSortFilterProxyModel, public QQmlParserStatus
 {
     Q_OBJECT
@@ -80,17 +83,6 @@ public:
     }
 };
 
-class KIOAccessManagerFactory : public QQmlNetworkAccessManagerFactory
-{
-public:
-    KIOAccessManagerFactory() = default;
-    ~KIOAccessManagerFactory() = default;
-    QNetworkAccessManager *create(QObject *parent) override
-    {
-        return new KIO::AccessManager(parent);
-    }
-};
-
 DiscoverObject::DiscoverObject(CompactMode mode)
     : QObject()
     , m_engine(new QQmlApplicationEngine)
@@ -102,7 +94,7 @@ DiscoverObject::DiscoverObject(CompactMode mode)
     auto factory = m_engine->networkAccessManagerFactory();
     m_engine->setNetworkAccessManagerFactory(nullptr);
     delete factory;
-    m_engine->setNetworkAccessManagerFactory(new KIOAccessManagerFactory());
+    m_engine->setNetworkAccessManagerFactory(m_networkAccessManagerFactory.data());
 
     qmlRegisterType<UnityLauncher>("org.kde.discover.app", 1, 0, "UnityLauncher");
     qmlRegisterType<PaginateModel>("org.kde.discover.app", 1, 0, "PaginateModel");
@@ -110,9 +102,19 @@ DiscoverObject::DiscoverObject(CompactMode mode)
     qmlRegisterType<FeaturedModel>("org.kde.discover.app", 1, 0, "FeaturedModel");
     qmlRegisterType<OurSortFilterProxyModel>("org.kde.discover.app", 1, 0, "QSortFilterProxyModel");
 
+    qmlRegisterSingletonType<DiscoverSettings>("org.kde.discover.app", 1, 0, "DiscoverSettings", [](QQmlEngine*, QJSEngine*) -> QObject* {
+            auto r = new DiscoverSettings;
+            connect(r, &DiscoverSettings::installedPageSortingChanged, r, &DiscoverSettings::save);
+            connect(r, &DiscoverSettings::appsListPageSortingChanged, r, &DiscoverSettings::save);
+            return r;
+        });
     qmlRegisterType<QQuickView>();
     qmlRegisterType<QActionGroup>();
     qmlRegisterType<QAction>();
+
+    qmlRegisterType<KAboutData>();
+    qmlRegisterType<KAboutLicense>();
+    qmlRegisterType<KAboutPerson>();
     qmlRegisterUncreatableType<DiscoverObject>("org.kde.discover.app", 1, 0, "DiscoverMainWindow", QStringLiteral("don't do that"));
     setupActions();
 
@@ -122,10 +124,8 @@ DiscoverObject::DiscoverObject(CompactMode mode)
     plugin->initializeEngine(m_engine, uri);
     plugin->registerTypes(uri);
 
-    //Here we set up a cache for the screenshots
-    delete m_engine->networkAccessManagerFactory();
-    m_engine->setNetworkAccessManagerFactory(m_networkAccessManagerFactory.data());
     m_engine->rootContext()->setContextProperty(QStringLiteral("app"), this);
+    m_engine->rootContext()->setContextProperty(QStringLiteral("discoverAboutData"), QVariant::fromValue(KAboutData::applicationData()));
 
     connect(m_engine, &QQmlApplicationEngine::objectCreated, this, &DiscoverObject::integrateObject);
     m_engine->load(QUrl(QStringLiteral("qrc:/qml/DiscoverWindow.qml")));
@@ -135,6 +135,22 @@ DiscoverObject::DiscoverObject(CompactMode mode)
         for(auto o: objs)
             delete o;
     });
+    auto action = new OneTimeAction(
+        [this]() {
+            bool found = DiscoverBackendsFactory::hasRequestedBackends();
+            for (auto b : ResourcesModel::global()->backends())
+                found |= b->hasApplications();
+
+            if (!found)
+                Q_EMIT openErrorPage(i18n("No application back-ends found, please report to your distribution."));
+        }
+        , this);
+
+    if (ResourcesModel::global()->backends().isEmpty()) {
+        connect(ResourcesModel::global(), &ResourcesModel::allInitialized, action, &OneTimeAction::trigger);
+    } else {
+        action->trigger();
+    }
 }
 
 DiscoverObject::~DiscoverObject()
@@ -220,21 +236,26 @@ void DiscoverObject::openLocalPackage(const QUrl& localfile)
     setRootObjectProperty("defaultStartup", false);
     auto action = new OneTimeAction(
         [this, localfile]() {
-            auto res = ResourcesModel::global()->resourceForFile(localfile);
-            qCDebug(DISCOVER_LOG) << "all initialized..." << res;
-            if (res) {
-                emit openApplicationInternal(res);
-            } else {
-                QMimeDatabase db;
-                auto mime = db.mimeTypeForUrl(localfile);
-                if (mime.name().startsWith(QLatin1String("application/vnd.flatpak"))) {
-                    openApplication(QUrl(QLatin1String("appstream://org.kde.discover.flatpak")));
-                    showPassiveNotification(i18n("Cannot interact with flatpak resources without the flatpak backend %1. Please install it first.", localfile.toDisplayString()));
+            AbstractResourcesBackend::Filters f;
+            f.resourceUrl = localfile;
+            auto stream = new StoredResultsStream({ResourcesModel::global()->search(f)});
+            connect(stream, &StoredResultsStream::finished, this, [this, localfile, stream]() {
+                const auto res = stream->resources();
+                if (res.count() == 1) {
+                    emit openApplicationInternal(res.first());
                 } else {
-                    setRootObjectProperty("defaultStartup", true);
-                    showPassiveNotification(i18n("Couldn't open %1", localfile.toDisplayString()));
+                    QMimeDatabase db;
+                    auto mime = db.mimeTypeForUrl(localfile);
+                    auto fIsFlatpakBackend = [](AbstractResourcesBackend* backend) { return backend->metaObject()->className() == QByteArray("FlatpakBackend"); };
+                    if (mime.name().startsWith(QLatin1String("application/vnd.flatpak")) && !kContains(ResourcesModel::global()->backends(), fIsFlatpakBackend)) {
+                        openApplication(QUrl(QStringLiteral("appstream://org.kde.discover.flatpak")));
+                        showPassiveNotification(i18n("Cannot interact with flatpak resources without the flatpak backend %1. Please install it first.", localfile.toDisplayString()));
+                    } else {
+                        setRootObjectProperty("defaultStartup", true);
+                        showPassiveNotification(i18n("Couldn't open %1", localfile.toDisplayString()));
+                    }
                 }
-            }
+            });
         }
         , this);
 
@@ -423,7 +444,7 @@ public:
         if (m_warnings.isEmpty())
             qCDebug(DISCOVER_LOG) << "cool no warnings!";
         else
-            qCDebug(DISCOVER_LOG) << "test finished succesfully despite" << m_warnings;
+            qCDebug(DISCOVER_LOG) << "test finished successfully despite" << m_warnings;
         qGuiApp->exit(m_warnings.count());
     }
 
