@@ -17,10 +17,10 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#ifndef MUONDISCOVERMAINWINDOW_H
-#define MUONDISCOVERMAINWINDOW_H
+#ifndef DISCOVEROBJECT_H
+#define DISCOVEROBJECT_H
 
-#include <QtCore/QUrl>
+#include <QUrl>
 
 #include <QQuickView>
 
@@ -97,4 +97,4 @@ class DiscoverObject : public QObject
         QScopedPointer<CachedNetworkAccessManagerFactory> m_networkAccessManagerFactory;
 };
 
-#endif // MUONINSTALLERDECLARATIVEVIEW_H
+#endif // DISCOVEROBJECT_H
