@@ -39,14 +39,17 @@
 #include <QTimer>
 #include <QStandardPaths>
 #include <QFile>
+#include <QAction>
+#include <QMimeDatabase>
+#include <QFileSystemWatcher>
 
 #include <PackageKit/Transaction>
 #include <PackageKit/Daemon>
+#include <PackageKit/Offline>
 #include <PackageKit/Details>
 
 #include <KLocalizedString>
-#include <QAction>
-#include <QMimeDatabase>
+#include <KProtocolManager>
 
 #include "utils.h"
 #include "config-paths.h"
@@ -85,13 +88,21 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
     t->start();
 
     m_delayedDetailsFetch.setSingleShot(true);
-    m_delayedDetailsFetch.setInterval(0);
+    m_delayedDetailsFetch.setInterval(100);
     connect(&m_delayedDetailsFetch, &QTimer::timeout, this, &PackageKitBackend::performDetailsFetch);
 
+    connect(PackageKit::Daemon::global(), &PackageKit::Daemon::restartScheduled, m_updater, &PackageKitUpdater::enableNeedsReboot);
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::updatesChanged, this, &PackageKitBackend::fetchUpdates);
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::isRunningChanged, this, &PackageKitBackend::checkDaemonRunning);
     connect(m_reviews.data(), &OdrsReviewsBackend::ratingsReady, this, &AbstractResourcesBackend::emitRatingsReady);
 
+    auto proxyWatch = new QFileSystemWatcher(this);
+    proxyWatch->addPath(QStandardPaths::writableLocation(QStandardPaths::ConfigLocation) + QLatin1String("/kioslaverc"));
+    connect(proxyWatch, &QFileSystemWatcher::fileChanged, this, [this](){
+        KProtocolManager::reparseConfiguration();
+        updateProxy();
+    });
+
     SourcesModel::global()->addSourcesBackend(new PackageKitSourcesBackend(this));
 
     reloadPackageList();
@@ -99,11 +110,31 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
     setWhenAvailable(PackageKit::Daemon::getTimeSinceAction(PackageKit::Transaction::RoleRefreshCache), [this](uint timeSince) {
         if (timeSince > 3600)
             checkForUpdates();
+        else
+            fetchUpdates();
     }, this);
 }
 
 PackageKitBackend::~PackageKitBackend() = default;
 
+void PackageKitBackend::updateProxy()
+{
+
+    if (PackageKit::Daemon::isRunning()) {
+        static bool everHad = KProtocolManager::useProxy();
+        if (!everHad && !KProtocolManager::useProxy())
+            return;
+
+        everHad = KProtocolManager::useProxy();
+        PackageKit::Daemon::global()->setProxy(KProtocolManager::proxyFor(QStringLiteral("http")),
+                                               KProtocolManager::proxyFor(QStringLiteral("https")),
+                                               KProtocolManager::proxyFor(QStringLiteral("ftp")),
+                                               KProtocolManager::proxyFor(QStringLiteral("socks")),
+                                               {},
+                                               {});
+    }
+}
+
 bool PackageKitBackend::isFetching() const
 {
     return m_isFetching;
@@ -216,14 +247,6 @@ AppPackageKitResource* PackageKitBackend::addComponent(const AppStream::Componen
     return res;
 }
 
-void PackageKitBackend::clearPackages(const QStringList& packageNames)
-{
-    const auto resources = resourcesByPackageNames<QVector<AbstractResource*>>(packageNames);
-    for(auto res: resources) {
-        qobject_cast<PackageKitResource*>(res)->clearPackageIds();
-    }
-}
-
 void PackageKitBackend::resolvePackages(const QStringList &packageNames)
 {
     PackageKit::Transaction * tArch = PackageKit::Daemon::resolve(packageNames, PackageKit::Transaction::FilterArch);
@@ -236,7 +259,6 @@ void PackageKitBackend::resolvePackages(const QStringList &packageNames)
 
     TransactionSet* merge = new TransactionSet({tArch, tNotArch});
     connect(merge, &TransactionSet::allFinished, this, &PackageKitBackend::getPackagesFinished);
-    fetchUpdates();
 }
 
 void PackageKitBackend::fetchUpdates()
@@ -358,6 +380,11 @@ T PackageKitBackend::resourcesByPackageNames(const QStringList &pkgnames) const
 
 void PackageKitBackend::checkForUpdates()
 {
+    if (PackageKit::Daemon::global()->offline()->updateTriggered()) {
+        qDebug() << "Won't be checking for updates again, the system needs a reboot to apply the fetched offline updates.";
+        return;
+    }
+
     if (!m_refresher) {
         acquireFetching(true);
         m_refresher = PackageKit::Daemon::refreshCache(false);
@@ -385,7 +412,7 @@ ResultsStream* PackageKitBackend::search(const AbstractResourcesBackend::Filters
         const auto ext = kTransform<QVector<AbstractResource*>>(m_packages.extendedBy[filter.extends], [](AppPackageKitResource* a){ return a; });
         return new ResultsStream(QStringLiteral("PackageKitStream-extends"), ext);
     } else if (filter.search.isEmpty()) {
-        return new ResultsStream(QStringLiteral("PackageKitStream-all"), kFilter<QVector<AbstractResource*>>(m_packages.packages, [](AbstractResource* res) { return !res->isTechnical(); }));
+        return new ResultsStream(QStringLiteral("PackageKitStream-all"), kFilter<QVector<AbstractResource*>>(m_packages.packages, [](AbstractResource* res) { return res->type() != AbstractResource::Technical; }));
     } else {
         const QList<AppStream::Component> components = m_appdata->search(filter.search);
         const QStringList ids = kTransform<QStringList>(components, [](const AppStream::Component& comp) { return comp.id(); });
@@ -420,7 +447,17 @@ ResultsStream* PackageKitBackend::search(const AbstractResourcesBackend::Filters
 ResultsStream * PackageKitBackend::findResourceByPackageName(const QUrl& url)
 {
     AbstractResource* pkg = nullptr;
-    if (url.host().isEmpty())
+    if (url.isLocalFile()) {
+        QMimeDatabase db;
+        const auto mime = db.mimeTypeForUrl(url);
+        if (    mime.inherits(QStringLiteral("application/vnd.debian.binary-package"))
+            || mime.inherits(QStringLiteral("application/x-rpm"))
+            || mime.inherits(QStringLiteral("application/x-tar"))
+            || mime.inherits(QStringLiteral("application/x-xz-compressed-tar"))
+        ) {
+            pkg = new LocalFilePKResource(url, this);
+        }
+    } else if (url.host().isEmpty())
         Q_EMIT passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
     else if (url.scheme() == QLatin1String("appstream")) {
         static const QMap<QString, QString> deprecatedAppstreamIds = {
@@ -460,6 +497,9 @@ bool PackageKitBackend::hasSecurityUpdates() const
 
 int PackageKitBackend::updatesCount() const
 {
+    if (PackageKit::Daemon::global()->offline()->updateTriggered())
+        return 0;
+
     int ret = 0;
     QSet<QString> packages;
     for(auto res: upgradeablePackages()) {
@@ -476,28 +516,19 @@ int PackageKitBackend::updatesCount() const
 Transaction* PackageKitBackend::installApplication(AbstractResource* app, const AddonList& addons)
 {
     Transaction* t = nullptr;
-    if(!addons.addonsToInstall().isEmpty())
-    {
-        QVector<AbstractResource*> appsToInstall;
-
+    if(!addons.addonsToInstall().isEmpty()) {
+        QVector<AbstractResource*> appsToInstall = resourcesByPackageNames<QVector<AbstractResource*>>(addons.addonsToInstall());
         if(!app->isInstalled())
             appsToInstall << app;
-
-        foreach(const QString& toInstall, addons.addonsToInstall()) {
-            appsToInstall += m_packages.packages.value(toInstall);
-            Q_ASSERT(appsToInstall.last());
-        }
         t = new PKTransaction(appsToInstall, Transaction::ChangeAddonsRole);
-    }
+    } else if (!app->isInstalled())
+        t = installApplication(app);
 
     if (!addons.addonsToRemove().isEmpty()) {
-        QVector<AbstractResource*> appsToRemove = kTransform<QVector<AbstractResource*>>(addons.addonsToRemove(), [this](const QString& toRemove){ return m_packages.packages.value(toRemove); });
+        const auto appsToRemove = resourcesByPackageNames<QVector<AbstractResource*>>(addons.addonsToRemove());
         t = new PKTransaction(appsToRemove, Transaction::RemoveRole);
     }
 
-    if (!app->isInstalled())
-        t = installApplication(app);
-
     return t;
 }
 
@@ -514,6 +545,10 @@ Transaction* PackageKitBackend::removeApplication(AbstractResource* app)
 
 QSet<AbstractResource*> PackageKitBackend::upgradeablePackages() const
 {
+    if (isFetching() || !m_packagesToAdd.isEmpty()) {
+        return {};
+    }
+
     QSet<AbstractResource*> ret;
     ret.reserve(m_updatesPackageId.size());
     Q_FOREACH (const QString& pkgid, m_updatesPackageId) {
@@ -543,10 +578,8 @@ void PackageKitBackend::addPackageToUpdate(PackageKit::Transaction::Info info, c
 void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
 {
     if (!m_updatesPackageId.isEmpty()) {
-        PackageKit::Transaction* transaction = PackageKit::Daemon::getDetails(m_updatesPackageId.toList());
-        connect(transaction, &PackageKit::Transaction::details, this, &PackageKitBackend::packageDetails);
-        connect(transaction, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
-        connect(transaction, &PackageKit::Transaction::finished, this, &PackageKitBackend::getUpdatesDetailsFinished);
+        resolvePackages(kTransform<QStringList>(m_updatesPackageId, [](const QString &pkgid) { return PackageKit::Daemon::packageName(pkgid); }));
+        fetchDetails(m_updatesPackageId);
     }
 
     m_updater->setProgressing(false);
@@ -555,13 +588,6 @@ void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
     emit updatesCountChanged();
 }
 
-void PackageKitBackend::getUpdatesDetailsFinished(PackageKit::Transaction::Exit exit, uint)
-{
-    if (exit != PackageKit::Transaction::ExitSuccess) {
-        qWarning() << "Couldn't figure out the updates on PackageKit backend" << exit;
-    }
-}
-
 bool PackageKitBackend::isPackageNameUpgradeable(const PackageKitResource* res) const
 {
     return !upgradeablePackageId(res).isEmpty();
@@ -577,7 +603,7 @@ QString PackageKitBackend::upgradeablePackageId(const PackageKitResource* res) c
     return QString();
 }
 
-void PackageKitBackend::fetchDetails(const QString& pkgid)
+void PackageKitBackend::fetchDetails(const QSet<QString>& pkgid)
 {
     if (!m_delayedDetailsFetch.isActive()) {
         m_delayedDetailsFetch.start();
@@ -594,13 +620,15 @@ void PackageKitBackend::performDetailsFetch()
     PackageKit::Transaction* transaction = PackageKit::Daemon::getDetails(ids);
     connect(transaction, &PackageKit::Transaction::details, this, &PackageKitBackend::packageDetails);
     connect(transaction, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
+    m_packageNamesToFetchDetails.clear();
 }
 
 void PackageKitBackend::checkDaemonRunning()
 {
     if (!PackageKit::Daemon::isRunning()) {
         qWarning() << "PackageKit stopped running!";
-    }
+    } else
+        updateProxy();
 }
 
 AbstractBackendUpdater* PackageKitBackend::backendUpdater() const
@@ -618,20 +646,6 @@ AbstractReviewsBackend* PackageKitBackend::reviewsBackend() const
     return m_reviews.data();
 }
 
-AbstractResource * PackageKitBackend::resourceForFile(const QUrl& file)
-{
-    QMimeDatabase db;
-    const auto mime = db.mimeTypeForUrl(file);
-    if (    mime.inherits(QLatin1String("application/vnd.debian.binary-package"))
-         || mime.inherits(QLatin1String("application/x-rpm"))
-         || mime.inherits(QLatin1String("application/x-tar"))
-         || mime.inherits(QLatin1String("application/x-xz-compressed-tar"))
-    ) {
-        return new LocalFilePKResource(file, this);
-    }
-    return nullptr;
-}
-
 static QString readDistroName()
 {
     const QStringList osreleasenames = (QStringList() << QStringLiteral("/etc/os-release")
